@@ -267,9 +267,6 @@ _gdk_input_common_find_events(GdkWindow *window,
   XEventClass class;
   
   i = 0;
-  /* We have to track press and release events in pairs to keep
-     track of button state correctly and implement grabbing for
-     the gxi support. FIXME - is this needed any more since gxi is gone? */
   if (mask & GDK_BUTTON_PRESS_MASK || mask & GDK_BUTTON_RELEASE_MASK)
     {
       DeviceButtonPress (gdkdev->xdevice, gdkdev->buttonpress_type,
@@ -279,10 +276,6 @@ _gdk_input_common_find_events(GdkWindow *window,
       DeviceButtonPressGrab (gdkdev->xdevice, 0, class);
       if (class != 0)
 	  classes[i++] = class;
-      DeviceButtonRelease (gdkdev->xdevice, gdkdev->buttonrelease_type,
-			   class);
-      if (class != 0)
-	  classes[i++] = class;
     }
   if (mask & GDK_POINTER_MOTION_MASK)
     {
