@@ -2,20 +2,20 @@
 
 # Build zipfiles for GTK on Win32
 
-ZIP=/tmp/gtk+-2.0.2-`date +%Y%m%d`.zip
-DEVZIP=/tmp/gtk+-dev-2.0.2-`date +%Y%m%d`.zip
-cd /opt
+ZIP=/tmp/gtk+-2.1.5-`date +%Y%m%d`.zip
+DEVZIP=/tmp/gtk+-dev-2.1.5-`date +%Y%m%d`.zip
+cd /usr
 
 rm $ZIP
 zip -r $ZIP -@ <<EOF
 COPYING.LIB-2
 etc/gtk-2.0
-lib/libgdk_pixbuf-2.0-0.dll
-lib/libgdk-win32-2.0-0.dll
-lib/libgtk-win32-2.0-0.dll
+lib/libgdk_pixbuf-2.1-0.dll
+lib/libgdk-win32-2.1-0.dll
+lib/libgtk-win32-2.1-0.dll
 EOF
 
-zip $ZIP lib/gtk-2.0/2.0.2/loaders/*.dll lib/gtk-2.0/immodules/2.0.2/*.dll
+zip $ZIP lib/gtk-2.0/2.2.0/loaders/*.dll lib/gtk-2.0/2.2.0/immodules/*.dll
 
 zip $ZIP share/themes/*/gtk-2.0/gtkrc share/themes/*/gtk-2.0-key/gtkrc
 
@@ -24,12 +24,12 @@ zip -r $ZIP lib/locale/*/LC_MESSAGES/gtk20.mo
 rm $DEVZIP
 zip -r $DEVZIP -@ <<EOF
 include/gtk-2.0
-lib/libgdk_pixbuf-2.0.dll.a
-lib/gdk_pixbuf-2.0.lib
-lib/libgdk-win32-2.0.dll.a
-lib/gdk-win32-2.0.lib
-lib/libgtk-win32-2.0.dll.a
-lib/gtk-win32-2.0.lib
+lib/libgdk_pixbuf-2.1.dll.a
+lib/gdk_pixbuf-2.1.lib
+lib/libgdk-win32-2.1.dll.a
+lib/gdk-win32-2.1.lib
+lib/libgtk-win32-2.1.dll.a
+lib/gtk-win32-2.1.lib
 lib/gtk-2.0/include
 lib/pkgconfig/gdk-pixbuf-2.0.pc
 lib/pkgconfig/gdk-2.0.pc
