@@ -32,8 +32,10 @@
 #include "gtkbindings.h"
 #include "gtklabel.h"
 #include "gtkmain.h"
+#include "gtkmarshalers.h"
 #include "gtkmenu.h"
 #include "gtkmenuitem.h"
+#include "gtktearoffmenuitem.h"
 #include "gtkwindow.h"
 #include "gtkhbox.h"
 #include "gtkvscrollbar.h"
@@ -51,10 +53,11 @@
 					 * extends below the submenu
 					 */
 
-#define MENU_SCROLL_STEP 10
+#define MENU_SCROLL_STEP1 8
+#define MENU_SCROLL_STEP2 15
 #define MENU_SCROLL_ARROW_HEIGHT 16
-#define MENU_SCROLL_FAST_ZONE 4
-#define MENU_SCROLL_TIMEOUT1 150
+#define MENU_SCROLL_FAST_ZONE 8
+#define MENU_SCROLL_TIMEOUT1 50
 #define MENU_SCROLL_TIMEOUT2 50
 
 typedef struct _GtkMenuAttachData	GtkMenuAttachData;
@@ -73,6 +76,11 @@ struct _GtkMenuPrivate
   gint y;
 };
 
+enum {
+  MOVE_SCROLL,
+  LAST_SIGNAL
+};
+
 enum {
   PROP_0,
   PROP_TEAROFF_TITLE
@@ -103,6 +111,10 @@ static gboolean gtk_menu_expose            (GtkWidget        *widget,
 					    GdkEventExpose   *event);
 static gboolean gtk_menu_key_press         (GtkWidget        *widget,
 					    GdkEventKey      *event);
+static gboolean gtk_menu_button_press      (GtkWidget        *widget,
+					    GdkEventButton   *event);
+static gboolean gtk_menu_button_release    (GtkWidget        *widget,
+					    GdkEventButton   *event);
 static gboolean gtk_menu_motion_notify     (GtkWidget        *widget,
 					    GdkEventMotion   *event);
 static gboolean gtk_menu_enter_notify      (GtkWidget        *widget,
@@ -111,8 +123,11 @@ static gboolean gtk_menu_leave_notify      (GtkWidget        *widget,
 					    GdkEventCrossing *event);
 static void     gtk_menu_scroll_to         (GtkMenu          *menu,
 					    gint              offset);
-static void     gtk_menu_stop_scrolling    (GtkMenu          *menu);
-static gboolean gtk_menu_scroll_timeout    (gpointer          data);
+
+static void     gtk_menu_stop_scrolling        (GtkMenu  *menu);
+static void     gtk_menu_remove_scroll_timeout (GtkMenu  *menu);
+static gboolean gtk_menu_scroll_timeout        (gpointer  data);
+
 static void     gtk_menu_scroll_item_visible (GtkMenuShell    *menu_shell,
 					      GtkWidget       *menu_item);
 static void     gtk_menu_select_item       (GtkMenuShell     *menu_shell,
@@ -130,8 +145,9 @@ static void     gtk_menu_style_set         (GtkWidget        *widget,
 					    GtkStyle         *previous_style);
 static gboolean gtk_menu_focus             (GtkWidget        *widget,
 					    GtkDirectionType direction);
-static gint     gtk_menu_get_popup_delay   (GtkMenuShell    *menu_shell);
-
+static gint     gtk_menu_get_popup_delay   (GtkMenuShell     *menu_shell);
+static void     gtk_menu_real_move_scroll  (GtkMenu          *menu,
+					    GtkScrollType     type);
 
 static void     gtk_menu_stop_navigating_submenu       (GtkMenu          *menu);
 static gboolean gtk_menu_stop_navigating_submenu_cb    (gpointer          user_data);
@@ -163,6 +179,8 @@ static void _gtk_menu_refresh_accel_paths (GtkMenu *menu,
 static GtkMenuShellClass *parent_class = NULL;
 static const gchar	 *attach_data_key = "gtk-menu-attach-data";
 
+static guint menu_signals[LAST_SIGNAL] = { 0 };
+
 GtkMenuPrivate *
 gtk_menu_get_private (GtkMenu *menu)
 {
@@ -229,6 +247,16 @@ gtk_menu_class_init (GtkMenuClass *class)
   gobject_class->set_property = gtk_menu_set_property;
   gobject_class->get_property = gtk_menu_get_property;
 
+  menu_signals[MOVE_SCROLL] =
+    _gtk_binding_signal_new ("move_scroll",
+			     G_OBJECT_CLASS_TYPE (object_class),
+			     G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			     G_CALLBACK (gtk_menu_real_move_scroll),
+			     NULL, NULL,
+			     _gtk_marshal_VOID__ENUM,
+			     G_TYPE_NONE, 1,
+			     GTK_TYPE_SCROLL_TYPE);
+  
   g_object_class_install_property (gobject_class,
                                    PROP_TEAROFF_TITLE,
                                    g_param_spec_string ("tearoff-title",
@@ -246,11 +274,14 @@ gtk_menu_class_init (GtkMenuClass *class)
   widget_class->show = gtk_menu_show;
   widget_class->expose_event = gtk_menu_expose;
   widget_class->key_press_event = gtk_menu_key_press;
+  widget_class->button_press_event = gtk_menu_button_press;
+  widget_class->button_release_event = gtk_menu_button_release;
   widget_class->motion_notify_event = gtk_menu_motion_notify;
   widget_class->show_all = gtk_menu_show_all;
   widget_class->hide_all = gtk_menu_hide_all;
   widget_class->enter_notify_event = gtk_menu_enter_notify;
   widget_class->leave_notify_event = gtk_menu_leave_notify;
+  widget_class->motion_notify_event = gtk_menu_motion_notify;
   widget_class->style_set = gtk_menu_style_set;
   widget_class->focus = gtk_menu_focus;
 
@@ -303,6 +334,46 @@ gtk_menu_class_init (GtkMenuClass *class)
 				"move_current", 1,
 				GTK_TYPE_MENU_DIRECTION_TYPE,
 				GTK_MENU_DIR_CHILD);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_Home, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_START);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_KP_Home, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_START);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_End, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_END);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_KP_End, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_END);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_Page_Up, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_PAGE_UP);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_KP_Page_Up, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_PAGE_UP);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_Page_Down, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_PAGE_DOWN);
+  gtk_binding_entry_add_signal (binding_set,
+				GDK_KP_Page_Down, 0,
+				"move_scroll", 1,
+				GTK_TYPE_SCROLL_TYPE,
+				GTK_SCROLL_PAGE_DOWN);
 
   gtk_settings_install_property (g_param_spec_boolean ("gtk-can-change-accels",
 						       _("Can change accelerators"),
@@ -406,7 +477,8 @@ gtk_menu_window_size_request (GtkWidget      *window,
 
   if (private->have_position)
     {
-      gint screen_height = gdk_screen_height ();
+      GdkScreen *screen = gtk_widget_get_screen (window);
+      gint screen_height = gdk_screen_get_height (screen);
 
       if (private->y + requisition->height > screen_height)
 	requisition->height = screen_height - private->y;
@@ -518,6 +590,31 @@ gtk_menu_finalize (GObject *object)
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
+static void
+menu_change_screen (GtkMenu   *menu,
+		    GdkScreen *new_screen)
+{
+  if (menu->torn_off)
+    {
+      gtk_window_set_screen (GTK_WINDOW (menu->tearoff_window), new_screen);
+      gtk_menu_position (menu);
+    }
+
+  gtk_window_set_screen (GTK_WINDOW (menu->toplevel), new_screen);
+}
+
+static void
+attach_widget_screen_changed (GtkWidget *attach_widget,
+			      GdkScreen *previous_screen,
+			      GtkMenu   *menu)
+{
+  if (gtk_widget_has_screen (attach_widget) &&
+      !g_object_get_data (G_OBJECT (menu), "gtk-menu-explicit-screen"))
+    {
+      menu_change_screen (menu, gtk_widget_get_screen (attach_widget));
+    }
+}
+
 void
 gtk_menu_attach_to_widget (GtkMenu	       *menu,
 			   GtkWidget	       *attach_widget,
@@ -537,7 +634,7 @@ gtk_menu_attach_to_widget (GtkMenu	       *menu,
     {
       g_warning ("gtk_menu_attach_to_widget(): menu already attached to %s",
 		 g_type_name (G_TYPE_FROM_INSTANCE (data->attach_widget)));
-      return;
+     return;
     }
   
   g_object_ref (menu);
@@ -545,6 +642,11 @@ gtk_menu_attach_to_widget (GtkMenu	       *menu,
   
   data = g_new (GtkMenuAttachData, 1);
   data->attach_widget = attach_widget;
+  
+  g_signal_connect (attach_widget, "screen_changed",
+		    G_CALLBACK (attach_widget_screen_changed), menu);
+  attach_widget_screen_changed (attach_widget, NULL, menu);
+  
   data->detacher = detacher;
   g_object_set_data (G_OBJECT (menu), attach_data_key, data);
   
@@ -589,6 +691,10 @@ gtk_menu_detach (GtkMenu *menu)
     }
   g_object_set_data (G_OBJECT (menu), attach_data_key, NULL);
   
+  g_signal_handlers_disconnect_by_func (data->attach_widget,
+					(gpointer) attach_widget_screen_changed,
+					menu);
+
   data->detacher (data->attach_widget, menu);
   
   if (GTK_WIDGET_REALIZED (menu))
@@ -747,7 +853,6 @@ gtk_menu_popup (GtkMenu		    *menu,
   GtkWidget *parent;
   GdkEvent *current_event;
   GtkMenuShell *menu_shell;
-  GtkMenuAttachData *attach_data;
 
   g_return_if_fail (GTK_IS_MENU (menu));
   
@@ -756,21 +861,6 @@ gtk_menu_popup (GtkMenu		    *menu,
   
   menu_shell->parent_menu_shell = parent_menu_shell;
   
-  if (!g_object_get_data (G_OBJECT (menu), "gtk-menu-explicit-screen"))
-    {
-      /* The screen was not set explicitly, if the menu is
-       * attached to a widget, try to get screen from its 
-       * toplevel window else go with the default
-       */
-      attach_data = g_object_get_data (G_OBJECT (menu), attach_data_key);
-      if (attach_data)
-	{
-	  if (!GTK_WIDGET_REALIZED (menu))
-	  gtk_window_set_screen (GTK_WINDOW (menu->toplevel),
-				 gtk_widget_get_screen (attach_data->attach_widget));
-	}
-    }
-
   /* Find the last viewable ancestor, and make an X grab on it
    */
   parent = GTK_WIDGET (menu);
@@ -1248,7 +1338,7 @@ gtk_menu_set_tearoff_state (GtkMenu  *menu,
 		GTK_ADJUSTMENT (gtk_adjustment_new (0,
 						    0,
 						    GTK_WIDGET (menu)->requisition.height,
-						    MENU_SCROLL_STEP,
+						    MENU_SCROLL_STEP2,
 						    height/2,
 						    height));
 	      g_object_connect (menu->tearoff_adjustment,
@@ -1289,6 +1379,12 @@ gtk_menu_set_tearoff_state (GtkMenu  *menu,
 	{
 	  gtk_widget_hide (menu->tearoff_window);
 	  gtk_menu_reparent (menu, menu->toplevel, FALSE);
+	  gtk_widget_destroy (menu->tearoff_window);
+	  
+	  menu->tearoff_window = NULL;
+	  menu->tearoff_hbox = NULL;
+	  menu->tearoff_scrollbar = NULL;
+	  menu->tearoff_adjustment = NULL;
 	}
     }
 }
@@ -1842,6 +1938,40 @@ gtk_menu_show (GtkWidget *widget)
   GTK_WIDGET_CLASS (parent_class)->show (widget);
 }
 
+static gboolean
+gtk_menu_button_press (GtkWidget      *widget,
+			 GdkEventButton *event)
+{
+  /* Don't pop down the menu for releases over scroll arrows
+   */
+  if (GTK_IS_MENU (widget))
+    {
+      GtkMenu *menu = GTK_MENU (widget);
+
+      if (menu->upper_arrow_prelight ||  menu->lower_arrow_prelight)
+	return TRUE;
+    }
+
+  return GTK_WIDGET_CLASS (parent_class)->button_press_event (widget, event);
+}
+
+static gboolean
+gtk_menu_button_release (GtkWidget      *widget,
+			 GdkEventButton *event)
+{
+  /* Don't pop down the menu for releases over scroll arrows
+   */
+  if (GTK_IS_MENU (widget))
+    {
+      GtkMenu *menu = GTK_MENU (widget);
+
+      if (menu->upper_arrow_prelight ||  menu->lower_arrow_prelight)
+	return TRUE;
+    }
+
+  return GTK_WIDGET_CLASS (parent_class)->button_release_event (widget, event);
+}
+
 static gboolean
 gtk_menu_key_press (GtkWidget	*widget,
 		    GdkEventKey *event)
@@ -2145,15 +2275,14 @@ gtk_menu_handle_scrolling (GtkMenu *menu, gboolean enter)
 	  /* Deselect the active item so that any submenus are poped down */
 	  gtk_menu_shell_deselect (menu_shell);
 
-	  gtk_menu_stop_scrolling (menu);
-	  menu->scroll_step = -MENU_SCROLL_STEP;
+	  gtk_menu_remove_scroll_timeout (menu);
+	  menu->scroll_step = (scroll_fast) ? -MENU_SCROLL_STEP2 : -MENU_SCROLL_STEP1;
 	  menu->timeout_id = g_timeout_add ((scroll_fast) ? MENU_SCROLL_TIMEOUT2 : MENU_SCROLL_TIMEOUT1,
 					    gtk_menu_scroll_timeout,
 					    menu);
 	}
       else if (!enter && !in_arrow && menu->upper_arrow_prelight)
 	{
-	  menu->upper_arrow_prelight = FALSE;
 	  gdk_window_invalidate_rect (GTK_WIDGET (menu)->window, &rect, FALSE);
 	  
 	  gtk_menu_stop_scrolling (menu);
@@ -2185,15 +2314,14 @@ gtk_menu_handle_scrolling (GtkMenu *menu, gboolean enter)
 	  /* Deselect the active item so that any submenus are poped down */
 	  gtk_menu_shell_deselect (menu_shell);
 
-	  gtk_menu_stop_scrolling (menu);
-	  menu->scroll_step = MENU_SCROLL_STEP;
+	  gtk_menu_remove_scroll_timeout (menu);
+	  menu->scroll_step = (scroll_fast) ? MENU_SCROLL_STEP2 : MENU_SCROLL_STEP1;
 	  menu->timeout_id = g_timeout_add ((scroll_fast) ? MENU_SCROLL_TIMEOUT2 : MENU_SCROLL_TIMEOUT1,
 					    gtk_menu_scroll_timeout,
 					    menu);
 	}
       else if (!enter && !in_arrow && menu->lower_arrow_prelight)
 	{
-	  menu->lower_arrow_prelight = FALSE;
 	  gdk_window_invalidate_rect (GTK_WIDGET (menu)->window, &rect, FALSE);
 	  
 	  gtk_menu_stop_scrolling (menu);
@@ -2208,8 +2336,13 @@ gtk_menu_enter_notify (GtkWidget        *widget,
   GtkWidget *menu_item;
 
   if (widget && GTK_IS_MENU (widget))
-    gtk_menu_handle_scrolling (GTK_MENU (widget), TRUE);
-      
+    {
+      GtkMenuShell *menu_shell = GTK_MENU_SHELL (widget);
+
+      if (!menu_shell->ignore_enter)
+	gtk_menu_handle_scrolling (GTK_MENU (widget), TRUE);
+    }
+  
   /* If this is a faked enter (see gtk_menu_motion_notify), 'widget'
    * will not correspond to the event widget's parent.  Check to see
    * if we are in the parent's navigation region.
@@ -2522,6 +2655,7 @@ gtk_menu_position (GtkMenu *menu)
   gint menu_height;
   gboolean push_in;
   GdkScreen *screen;
+  GdkScreen *pointer_screen;
   GdkRectangle monitor;
   gint monitor_num;
 
@@ -2529,14 +2663,9 @@ gtk_menu_position (GtkMenu *menu)
 
   widget = GTK_WIDGET (menu);
 
-  gdk_window_get_pointer (gtk_widget_get_root_window (widget),
-  			  &x, &y, NULL);
-
   screen = gtk_widget_get_screen (widget);
-  monitor_num = gdk_screen_get_monitor_at_point (screen, x, y);
-  if (monitor_num < 0)
-    monitor_num = 0;
-  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+  gdk_display_get_pointer (gdk_screen_get_display (screen),
+			   &pointer_screen, &x, &y, NULL);
 
   /* We need the requisition to figure out the right place to
    * popup the menu. In fact, we always need to ask here, since
@@ -2545,6 +2674,20 @@ gtk_menu_position (GtkMenu *menu)
    */
   gtk_widget_size_request (widget, &requisition);
 
+  if (pointer_screen != screen)
+    {
+      /* Pointer is on a different screen; roughly center the
+       * menu on the screen. If someone was using multiscreen
+       * + Xinerama together they'd probably want something
+       * fancier; but that is likely to be vanishingly rare.
+       */
+      x = MAX (0, (gdk_screen_get_width (screen) - requisition.width) / 2);
+      y = MAX (0, (gdk_screen_get_height (screen) - requisition.height) / 2);
+    }
+
+  monitor_num = gdk_screen_get_monitor_at_point (screen, x, y);
+  gdk_screen_get_monitor_geometry (screen, monitor_num, &monitor);
+
   push_in = FALSE;
   
   if (menu->position_func)
@@ -2612,7 +2755,7 @@ gtk_menu_position (GtkMenu *menu)
 }
 
 static void
-gtk_menu_stop_scrolling (GtkMenu *menu)
+gtk_menu_remove_scroll_timeout (GtkMenu *menu)
 {
   if (menu->timeout_id)
     {
@@ -2621,6 +2764,14 @@ gtk_menu_stop_scrolling (GtkMenu *menu)
     }
 }
 
+static void
+gtk_menu_stop_scrolling (GtkMenu *menu)
+{
+  gtk_menu_remove_scroll_timeout (menu);
+
+  menu->upper_arrow_prelight = FALSE;
+  menu->lower_arrow_prelight = FALSE;
+}
 
 static void
 gtk_menu_scroll_to (GtkMenu *menu,
@@ -2639,7 +2790,9 @@ gtk_menu_scroll_to (GtkMenu *menu,
       menu->tearoff_adjustment &&
       (menu->tearoff_adjustment->value != offset))
     {
-      menu->tearoff_adjustment->value = offset;
+      menu->tearoff_adjustment->value =
+	CLAMP (offset,
+	       0, menu->tearoff_adjustment->upper - menu->tearoff_adjustment->page_size);
       gtk_adjustment_value_changed (menu->tearoff_adjustment);
     }
   
@@ -2654,11 +2807,11 @@ gtk_menu_scroll_to (GtkMenu *menu,
 
   x = border_width + widget->style->xthickness;
   y = border_width + widget->style->ythickness;
-  
+
   if (!menu->tearoff_active)
     {
       last_visible = menu->upper_arrow_visible;
-      menu->upper_arrow_visible = (offset > 0);
+      menu->upper_arrow_visible = (view_height < menu_height && offset > 0);
       
       if (menu->upper_arrow_visible)
 	view_height -= MENU_SCROLL_ARROW_HEIGHT;
@@ -2672,9 +2825,9 @@ gtk_menu_scroll_to (GtkMenu *menu,
 	  if (menu->scroll_step < 0)
 	    gtk_menu_stop_scrolling (menu);
 	}
-      
+
       last_visible = menu->lower_arrow_visible;
-      menu->lower_arrow_visible = (view_height + offset < menu_height);
+      menu->lower_arrow_visible = (view_height < menu_height && offset < menu_height - view_height);
       
       if (menu->lower_arrow_visible)
 	view_height -= MENU_SCROLL_ARROW_HEIGHT;
@@ -2709,14 +2862,54 @@ gtk_menu_scroll_to (GtkMenu *menu,
   menu->scroll_offset = offset;
 }
 
+static gboolean
+compute_child_offset (GtkMenu   *menu,
+		      GtkWidget *menu_item,
+		      gint      *offset,
+		      gint      *height,
+		      gboolean  *is_last_child)
+{
+  GtkMenuShell *menu_shell = GTK_MENU_SHELL (menu);
+  GList *children;
+  gint child_offset = 0;
+
+  for (children = menu_shell->children; children; children = children->next)
+    {
+      GtkWidget *child = children->data;
+      GtkRequisition child_requisition;
+      gint child_height;
+      
+      if (GTK_WIDGET_VISIBLE (child))
+	{
+	  gtk_widget_size_request (child, &child_requisition);
+	  child_height = child_requisition.height;
+	}
+      else
+	child_height = 0;
+      
+      if (child == menu_item)
+	{
+	  if (is_last_child)
+	    *is_last_child = (children == NULL);
+	  if (offset)
+	    *offset = child_offset;
+	  if (height)
+	    *height = child_height;
+	  
+	  return TRUE;
+	}
+      
+      child_offset += child_height;
+    }
+
+  return FALSE;
+}
+
 static void
 gtk_menu_scroll_item_visible (GtkMenuShell    *menu_shell,
 			      GtkWidget       *menu_item)
 {
   GtkMenu *menu;
-  GtkWidget *child;
-  GList *children;
-  GtkRequisition child_requisition;
   gint child_offset, child_height;
   gint width, height;
   gint y;
@@ -2730,37 +2923,15 @@ gtk_menu_scroll_item_visible (GtkMenuShell    *menu_shell,
    * visible.
    */
 
-  child = NULL;
-  child_offset = 0;
-  child_height = 0;
-  children = menu_shell->children;
-  while (children)
-    {
-      child = children->data;
-      children = children->next;
-      
-      if (GTK_WIDGET_VISIBLE (child))
-	{
-	  gtk_widget_size_request (child, &child_requisition);
-	  child_offset += child_height;
-	  child_height = child_requisition.height;
-	}
-      
-      if (child == menu_item)
-	{
-	  last_child = (children == NULL);
-	  break;
-	}
-    }
-
-  if (child == menu_item)
+  if (compute_child_offset (menu, menu_item,
+			    &child_offset, &child_height, &last_child))
     {
       y = menu->scroll_offset;
       gdk_drawable_get_size (GTK_WIDGET (menu)->window, &width, &height);
 
       height -= 2*GTK_CONTAINER (menu)->border_width + 2*GTK_WIDGET (menu)->style->ythickness;
       
-      if (child_offset + child_height <= y)
+      if (child_offset < y)
 	{
 	  /* Ignore the enter event we might get if the pointer is on the menu
 	   */
@@ -2775,7 +2946,7 @@ gtk_menu_scroll_item_visible (GtkMenuShell    *menu_shell,
 	  if (menu->lower_arrow_visible && !menu->tearoff_active)
 	    arrow_height += MENU_SCROLL_ARROW_HEIGHT;
 	  
-	  if (child_offset >= y + height - arrow_height)
+	  if (child_offset + child_height > y + height - arrow_height)
 	    {
 	      arrow_height = 0;
 	      if (!last_child && !menu->tearoff_active)
@@ -2891,19 +3062,32 @@ gtk_menu_hide_all (GtkWidget *widget)
 /**
  * gtk_menu_set_screen:
  * @menu: a #GtkMenu.
- * @screen: a #GtkScreen.
+ * @screen: a #GdkScreen, or %NULL if the screen should be
+ *          determined by the widget the menu is attached to.
  *
- * Sets the #GtkScreen on which the GtkMenu will be displayed.
- * This function can only be called before @menu is realized.
+ * Sets the #GdkScreen on which the menu will be displayed.
+ * 
+ * Since: 2.2
  **/
 void
-gtk_menu_set_screen (GtkMenu *menu, 
+gtk_menu_set_screen (GtkMenu   *menu, 
 		     GdkScreen *screen)
 {
   g_return_if_fail (GTK_IS_MENU (menu));
-  gtk_window_set_screen (GTK_WINDOW (menu->toplevel), 
-			 screen);
+  g_return_if_fail (!screen || GDK_IS_SCREEN (screen));
+
   g_object_set_data (G_OBJECT (menu), "gtk-menu-explicit-screen", screen);
+
+  if (screen)
+    {
+      menu_change_screen (menu, screen);
+    }
+  else
+    {
+      GtkWidget *attach_widget = gtk_menu_get_attach_widget (menu);
+      if (attach_widget)
+	attach_widget_screen_changed (attach_widget, NULL, menu);
+    }
 }
 
 
@@ -2918,3 +3102,136 @@ gtk_menu_get_popup_delay (GtkMenuShell *menu_shell)
 
   return popup_delay;
 }
+
+static gint
+get_visible_size (GtkMenu *menu)
+{
+  GtkWidget *widget = GTK_WIDGET (menu);
+  GtkContainer *container = GTK_CONTAINER (menu);
+  
+  gint menu_height = (widget->allocation.height
+		      - 2 * (container->border_width
+			     + widget->style->ythickness));
+  
+  if (menu->upper_arrow_visible && !menu->tearoff_active)
+    menu_height -= MENU_SCROLL_ARROW_HEIGHT;
+  if (menu->lower_arrow_visible && !menu->tearoff_active)
+    menu_height -= MENU_SCROLL_ARROW_HEIGHT;
+
+  return menu_height;
+}
+
+/* Find the sensitive on-screen child containing @y, or if none,
+ * the nearest selectable onscreen child. (%NULL if none)
+ */
+static GtkWidget *
+child_at (GtkMenu *menu,
+	  gint     y)
+{
+  GtkMenuShell *menu_shell = GTK_MENU_SHELL (menu);
+  GtkWidget *child = NULL;
+  gint child_offset = 0;
+  GList *children;
+  gint menu_height;
+  gint lower, upper;		/* Onscreen bounds */
+
+  menu_height = get_visible_size (menu);
+  lower = menu->scroll_offset;
+  upper = menu->scroll_offset + menu_height;
+  
+  for (children = menu_shell->children; children; children = children->next)
+    {
+      if (GTK_WIDGET_VISIBLE (children->data))
+	{
+	  GtkRequisition child_requisition;
+
+	  gtk_widget_size_request (children->data, &child_requisition);
+
+	  if (_gtk_menu_item_is_selectable (children->data) &&
+	      child_offset >= lower &&
+	      child_offset + child_requisition.height <= upper)
+	    {
+	      child = children->data;
+	      
+	      if (child_offset + child_requisition.height > y &&
+		  !GTK_IS_TEAROFF_MENU_ITEM (child))
+		return child;
+	    }
+      
+	  child_offset += child_requisition.height;
+	}
+    }
+
+  return child;
+}
+
+static void
+gtk_menu_real_move_scroll (GtkMenu       *menu,
+			   GtkScrollType  type)
+{
+  GtkMenuShell *menu_shell = GTK_MENU_SHELL (menu);
+  
+  switch (type)
+    {
+    case GTK_SCROLL_PAGE_UP:
+    case GTK_SCROLL_PAGE_DOWN:
+      {
+	gint page_size = get_visible_size (menu);
+	gint old_offset;
+	gint child_offset = 0;
+	gboolean old_upper_arrow_visible;
+	gint step;
+
+	if (type == GTK_SCROLL_PAGE_UP)
+	  step = - page_size;
+	else
+	  step = page_size;
+
+	if (menu_shell->active_menu_item)
+	  {
+	    gint child_height;
+	    
+	    compute_child_offset (menu, menu_shell->active_menu_item,
+				  &child_offset, &child_height, NULL);
+	    child_offset += child_height / 2;
+	  }
+
+	menu_shell->ignore_enter = TRUE;
+	old_upper_arrow_visible = menu->upper_arrow_visible && !menu->tearoff_active;
+	old_offset = menu->scroll_offset;
+	gtk_menu_scroll_to (menu, menu->scroll_offset + step);
+	
+	if (menu_shell->active_menu_item)
+	  {
+	    GtkWidget *new_child;
+	    gboolean new_upper_arrow_visible = menu->upper_arrow_visible && !menu->tearoff_active;
+
+	    if (menu->scroll_offset != old_offset)
+	      step = menu->scroll_offset - old_offset;
+
+	    step -= (new_upper_arrow_visible - old_upper_arrow_visible) * MENU_SCROLL_ARROW_HEIGHT;
+
+	    new_child = child_at (menu, child_offset + step);
+	    if (new_child)
+	      gtk_menu_shell_select_item (menu_shell, new_child);
+	  }
+      }
+      break;
+    case GTK_SCROLL_START:
+      /* Ignore the enter event we might get if the pointer is on the menu
+       */
+      menu_shell->ignore_enter = TRUE;
+      gtk_menu_scroll_to (menu, 0);
+      gtk_menu_shell_select_first (menu_shell, TRUE);
+      break;
+    case GTK_SCROLL_END:
+      /* Ignore the enter event we might get if the pointer is on the menu
+       */
+      menu_shell->ignore_enter = TRUE;
+      gtk_menu_scroll_to (menu, G_MAXINT);
+      _gtk_menu_shell_select_last (menu_shell, TRUE);
+      break;
+    default:
+      break;
+    }
+}
